#include <kdialogbase.h>
#include <klocale.h>
#include <qvgroupbox.h>
#include <klistview.h>
#include <qsocket.h>
#include <qpopupmenu.h>
#include <kdebug.h>
#include <qtimer.h>
#include <qwhatsthis.h>

#include "server.h"
#include "status.h"
#include "plugin.h"

HostItem::HostItem(KListView *parent, const QHostAddress &a) : KListViewItem(parent) {
	_address = a;

	_async_lookup = new QDns(_address);
	connect(_async_lookup, SIGNAL(resultsReady()), this, SLOT(lookupComplete()));

	_connect_time = QDateTime::currentDateTime();

	_update_timer = new QTimer();
	connect(_update_timer, SIGNAL(timeout()), this, SLOT(updateTime()));
	_update_timer->start(60000);	

	setText(0, a.toString());
	updateTime();
}

HostItem::~HostItem() {
	delete _async_lookup;
	delete _update_timer;
}

void HostItem::lookupComplete() {
	setText(0, _async_lookup->canonicalName());
}

void HostItem::updateTime() {
	unsigned int hours, minutes, seconds;
	QString time_string;

	seconds = _connect_time.secsTo(QDateTime::currentDateTime());

	minutes = (seconds % 3600) / 60;
	hours = seconds / 3600;

	time_string = QString::number(hours) + ":";

	if (minutes < 10)
		time_string += "0";

	time_string += QString::number(minutes);

	setText(1, time_string);
}

FloodcastStatus::FloodcastStatus(FloodcastPlugin *daddy) : KDialogBase(0, "FloodcastStatus", false,
                                                                       i18n("Floodcast Status"), Close)
{
	_daddy = daddy;

	connect(this, SIGNAL(closeClicked()), SLOT(hide()));
	connect(daddy->server(), SIGNAL(gainedConnection(QSocket *)), this, SLOT(gainedConnection(QSocket *)));
	connect(daddy->server(), SIGNAL(lostConnection(QSocket *)), this, SLOT(lostConnection(QSocket *)));

	buildUI();

	_context_menu = new QPopupMenu(this);
	_context_menu->insertItem(i18n("&Disconnect"), this, SLOT(disconnectClient()));
}

void FloodcastStatus::buildUI() {
	QVGroupBox *groupBox = new QVGroupBox(i18n("Connected Users"), this);
	groupBox->setMinimumWidth(380);
	setMainWidget(groupBox);

	_list_view = new KListView(groupBox);
	_list_view->addColumn(i18n("Hostname"));
	_list_view->addColumn(i18n("Duration"));
	_list_view->setSelectionMode(QListView::Extended);

	QWhatsThis::add(_list_view, i18n("A list of currently connected users, and how long they have been connected "
	                                 "in minutes. You can forcibly disconnect a user by right clicking on their "
	                                 "list entry selecting \"Disconnect\"."));
	connect(_list_view, SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)), this, SLOT(showContextMenu(QListViewItem *, const QPoint &)));
}


void FloodcastStatus::gainedConnection(QSocket *s) {
	// Make a listview item
	HostItem *list_item = new HostItem(_list_view, s->peerAddress());
	_entry_map[s] = list_item;
}

void FloodcastStatus::lostConnection(QSocket *s) {
	if (_entry_map.contains(s)) {
		delete _entry_map[s];
		_entry_map.remove(s);
	}
}

void FloodcastStatus::disconnectClient() {
	QMap<QSocket *, HostItem*>::iterator listview_it;
	QValueList<QSocket*>::iterator sockets_it;
	QValueList<QSocket*> dead_sockets;

	for (listview_it = _entry_map.begin(); listview_it != _entry_map.end(); ++listview_it) {
		if (listview_it.data()->isSelected()) {
			dead_sockets.push_back(listview_it.key());
		}
	}

	for (sockets_it = dead_sockets.begin();sockets_it != dead_sockets.end(); ++sockets_it) {
		_daddy->server()->removeSocket(*sockets_it);
	}
}

void FloodcastStatus::showContextMenu(QListViewItem *, const QPoint &pos) {
	_context_menu->exec(pos);
}


#include "status.moc"
